﻿// DelegateCommand.cs
//

using System;
using System.Windows;
using System.Windows.Input;

namespace HelloWorld {

    public class DelegateCommand : ICommand {

        private EventHandler _canExecuteChangedHandler;

        private readonly Action _executeMethod;
        private readonly Func<bool> _canExecuteMethod;

        public DelegateCommand(Action executeMethod)
            : this(executeMethod, null) {
        }

        public DelegateCommand(Action executeMethod, Func<bool> canExecuteMethod) {
            if (executeMethod == null) {
                throw new ArgumentNullException("executeMethod");
            }

            _executeMethod = executeMethod;
            _canExecuteMethod = canExecuteMethod;
        }

        public bool CanExecute() {
            if (_canExecuteMethod == null) {
                return true;
            }
            return _canExecuteMethod();
        }

        public void Execute(object parameter) {
            _executeMethod();
        }

        public void RaiseCanExecuteChanged() {
            OnCanExecuteChanged();
        }

        protected virtual void OnCanExecuteChanged() {
            if (_canExecuteChangedHandler != null) {
                _canExecuteChangedHandler(this, EventArgs.Empty);
            }
        }

        #region ICommand Implementation
        event EventHandler ICommand.CanExecuteChanged {
            add {
                _canExecuteChangedHandler = (EventHandler)Delegate.Combine(_canExecuteChangedHandler, value);
            }
            remove {
                _canExecuteChangedHandler = (EventHandler)Delegate.Remove(_canExecuteChangedHandler, value);
            }
        }

        bool ICommand.CanExecute(object parameter) {
            return CanExecute();
        }

        void ICommand.Execute(object parameter) {
            Execute(parameter);
        }
        #endregion
    }
}
