﻿// HelloWorldViewModel.cs
//

using System;
using System.ComponentModel;
using System.Windows.Input;

namespace HelloWorld {

    public class HelloWorldViewModel : INotifyPropertyChanged {

        private string _name;
        private bool _happy = true;
        private string _greeting = "Hello";

        public string Name {
            get {
                return _name;
            }
            set {
                _name = value;
                RaisePropertyChanged("Name");
            }
        }

        public bool IsHappy {
            get {
                return _happy;
            }
            set {
                _happy = value;
                RaisePropertyChanged("IsHappy");
            }
        }

        public string Greeting {
            get {
                return _greeting;
            }
            private set {
                _greeting = value;
                RaisePropertyChanged("Greeting");
            }
        }

        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;

        #endregion

        private void RaisePropertyChanged(string propertyName) {
            if (PropertyChanged != null) {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }

        public void Hello() {
            if (String.IsNullOrEmpty(Name)) {
                Greeting = "Hello World!";
            }
            else {
                string greetingFormat;
                if (IsHappy == true) {
                    greetingFormat = "Hey! Thanks for stopping by {0}!";
                }
                else {
                    greetingFormat = "Sorry to hear that, {0}. Hope things turn around soon!";
                }

                Greeting = String.Format(greetingFormat, Name);
            }
        }

        private DelegateCommand _helloCommand;
        public ICommand HelloCommand {
            get {
                if (_helloCommand == null) {
                    _helloCommand = new DelegateCommand(Hello);
                }
                return _helloCommand;
            }
        }
    }
}
