﻿// DelegateCommand.cs
//

using System;
using System.Windows;
using System.Windows.Input;

namespace PatternsFramework {

    public class DelegateCommand : ICommand {

        private readonly Action _executeMethod;
        private readonly Func<bool> _canExecuteMethod;

        public DelegateCommand(Action executeMethod)
            : this(executeMethod, null) {
        }

        public DelegateCommand(Action executeMethod, Func<bool> canExecuteMethod) {
            if (executeMethod == null) {
                throw new ArgumentNullException("executeMethod");
            }

            _executeMethod = executeMethod;
            _canExecuteMethod = canExecuteMethod;
        }

        public bool CanExecute() {
            if (_canExecuteMethod == null) {
                return true;
            }
            return _canExecuteMethod();
        }

        public void Execute(object parameter) {
            if (_executeMethod == null) {
                return;
            }
            _executeMethod();
        }

        bool ICommand.CanExecute(object parameter) {
            return CanExecute();
        }

        public event EventHandler CanExecuteChanged;

        void ICommand.Execute(object parameter) {
            Execute(parameter);
        }

        public void RaiseCanExecuteChanged() {
            OnCanExecuteChanged();
        }

        protected virtual void OnCanExecuteChanged() {
            EventHandler canExecuteChangedHandler = CanExecuteChanged;
            if (canExecuteChangedHandler != null) {
                canExecuteChangedHandler(this, EventArgs.Empty);
            }
        }
    }
}
