﻿// EventAggregator.cs
//

using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using EventHandlerMap = System.Collections.Generic.Dictionary<object, object>;

namespace PatternsFramework {

    [Export(typeof(IEventAggregator))]
    public sealed class EventAggregator : IEventAggregator {

        private Dictionary<Type, EventHandlerMap> _subscriptionMap;

        public EventAggregator() {
            _subscriptionMap = new Dictionary<Type, EventHandlerMap>();
        }

        public void Publish<T>(T eventData) where T : EventArgs {
            if (eventData == null) {
                throw new ArgumentNullException("eventData");
            }

            EventHandlerMap eventHandlers;
            if (_subscriptionMap.TryGetValue(typeof(T), out eventHandlers)) {
                foreach (Action<T> handler in eventHandlers.Values) {
                    handler(eventData);
                }
            }
        }

        public object Subscribe<T>(Action<T> eventHandler) where T : EventArgs {
            if (eventHandler == null) {
                throw new ArgumentNullException("eventHandler");
            }

            object subscription = new object();

            EventHandlerMap eventHandlers;
            if (_subscriptionMap.TryGetValue(typeof(T), out eventHandlers) == false) {
                eventHandlers = new EventHandlerMap();
                _subscriptionMap[typeof(T)] = eventHandlers;
            }

            eventHandlers[subscription] = eventHandler;
            return subscription;
        }

        public void Unsubscribe<T>(object subscription) {
            if (subscription == null) {
                throw new ArgumentNullException("subscription");
            }

            EventHandlerMap eventHandlers;
            if (_subscriptionMap.TryGetValue(typeof(T), out eventHandlers)) {
                eventHandlers.Remove(subscription);
            }
        }
    }
}
