﻿// IocContainer.cs
//

using System;
using System.ComponentModel;
using System.ComponentModel.Composition;
using System.Windows;
using System.ComponentModel.Composition.Hosting;

namespace PatternsFramework {

    public sealed class IocContainer : IApplicationService, IApplicationLifetimeAware {

        private CompositionContainer _container;

        public void InitializeContainer() {
            PackageCatalog catalog = new PackageCatalog();
            catalog.AddPackage(Package.Current);

            _container = new CompositionContainer(catalog);
        }

        public T GetObject<T>() where T : class {
            Lazy<T> export = _container.GetExport<T>();
            if (export != null) {
                return export.Value;
            }

            return null;
        }

        #region IApplicationService Members

        void IApplicationService.StartService(ApplicationServiceContext context) {
        }

        void IApplicationService.StopService() {
        }

        #endregion

        #region IApplicationLifetimeAware Members

        void IApplicationLifetimeAware.Exited() {
        }

        void IApplicationLifetimeAware.Exiting() {
        }

        void IApplicationLifetimeAware.Started() {
        }

        void IApplicationLifetimeAware.Starting() {
            InitializeContainer();
        }

        #endregion
    }
}
