﻿// ModelEventTrigger.cs
//

using System;
using System.Reflection;
using System.Windows;

namespace PatternsFramework {

    public class ModelEventTrigger : System.Windows.Interactivity.TriggerBase<FrameworkElement> {

        private Delegate _eventHandler;

        public string EventName {
            get;
            set;
        }

        protected override void OnAttached() {
            object source = AssociatedObject.DataContext;

            if (source != null) {
                Type sourceType = source.GetType();
                string eventName = EventName;
                EventInfo eventInfo = sourceType.GetEvent(eventName, BindingFlags.Public | BindingFlags.Instance |
                                                                     BindingFlags.FlattenHierarchy);

                if (eventInfo != null) {
                    MethodInfo eventHandlerMethod =
                        typeof(ModelEventTrigger).GetMethod("OnEvent", BindingFlags.NonPublic | BindingFlags.Instance |
                                                            BindingFlags.FlattenHierarchy);
                    _eventHandler = Delegate.CreateDelegate(eventInfo.EventHandlerType, this, eventHandlerMethod, /* throwOnBindFailure */ false);
                    if (_eventHandler != null) {
                        eventInfo.AddEventHandler(source, _eventHandler);
                    }
                }
            }

            base.OnAttached();
        }

        protected override void OnDetaching() {
            if (_eventHandler != null) {
                object source = AssociatedObject.DataContext;

                Type sourceType = source.GetType();
                EventInfo eventInfo = sourceType.GetEvent(EventName, BindingFlags.Public | BindingFlags.Instance |
                                                                     BindingFlags.FlattenHierarchy);

                eventInfo.RemoveEventHandler(source, _eventHandler);
                _eventHandler = null;
            }
            base.OnDetaching();
        }

        private void OnEvent(object sender, EventArgs e) {
            InvokeActions(e);
        }
    }
}
