﻿// StringFormatter.cs
//

using System;
using System.Globalization;
using System.Windows;
using System.Windows.Data;

namespace PatternsFramework {

    /// <summary>
    /// A value converter that can be used in a binding to generate
    /// a formatted string representation.
    /// </summary>
    public sealed class StringFormatter : IValueConverter {

        #region Implementation of IValueConverter
        object IValueConverter.Convert(object value, Type targetType, object parameter, CultureInfo culture) {
            if (targetType != typeof(string)) {
                throw new ArgumentOutOfRangeException("targetType", "StringFormatter can only convert to String");
            }

            if (value == null) {
                return String.Empty;
            }

            string format = parameter as string;
            if (String.IsNullOrEmpty(format) == false) {
                return String.Format(culture, format, value);
            }

            return String.Format(culture, "{0}", value);
        }

        object IValueConverter.ConvertBack(object value, Type targetType, object parameter, CultureInfo culture) {
            throw new NotImplementedException();
        }
        #endregion
    }
}
