﻿// App.xaml.cs
//

using System;
using System.Diagnostics;
using System.Threading;
using System.Windows;
using System.Windows.Browser;
using Microsoft.Silverlight.Testing;

namespace TwitterBug.Tests {

    public partial class App : Application {

        private SynchronizationContext _syncContext;

        public App() {
            _syncContext = SynchronizationContext.Current;

            this.Startup += this.Application_Startup;
            this.UnhandledException += this.Application_UnhandledException;

            InitializeComponent();
        }

        public new static App Current {
            get {
                return (App)Application.Current;
            }
        }

        public SynchronizationContext SyncContext {
            get {
                return _syncContext;
            }
        }

        private void Application_Startup(object sender, StartupEventArgs e) {
            RootVisual = UnitTestSystem.CreateTestPage();
        }

        private void Application_UnhandledException(object sender, ApplicationUnhandledExceptionEventArgs e) {
            // If the app is running outside of the debugger then report the exception using
            // the browser's exception mechanism. On IE this will display it a yellow alert 
            // icon in the status bar and Firefox will display a script error.
            if (!Debugger.IsAttached) {

                // NOTE: This will allow the application to continue running after an exception has been thrown
                // but not handled. 
                // For production applications this error handling should be replaced with something that will 
                // report the error to the website and stop the application.
                e.Handled = true;
                Deployment.Current.Dispatcher.BeginInvoke(delegate {
                    ReportErrorToDOM(e);
                });
            }
        }

        private void ReportErrorToDOM(ApplicationUnhandledExceptionEventArgs e) {
            try {
                string errorMsg = e.ExceptionObject.Message + e.ExceptionObject.StackTrace;
                errorMsg = errorMsg.Replace('"', '\'').Replace("\r\n", @"\n");

                HtmlPage.Window.Eval("throw new Error(\"Unhandled Error in Silverlight Application " + errorMsg + "\");");
            }
            catch (Exception) {
            }
        }
    }
}
