﻿// TweetBoxTests.cs
//

using System;
using Microsoft.Silverlight.Testing;
using Microsoft.Silverlight.Testing.UnitTesting.Metadata.VisualStudio;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using PatternsFramework;
using TwitterBug.Models;
using TwitterBug.Services;
using TwitterBug.Mocks;

namespace TwitterBug.Tests {

    [TestClass]
    public class TweetBoxTests : SilverlightTest {

        [TestMethod]
        public void TestNormalTweet() {
            ITwitterService twitterService = new MockTwitterService();
            IEventAggregator eventAggregator = new MockEventAggregator();

            TweetBoxViewModel vm = new TweetBoxViewModel(twitterService, eventAggregator);

            vm.Tweet = "Test Tweet Status Update";
            Assert.IsTrue(vm.RemainingLength > 0);
            Assert.IsTrue(vm.PostCommand.CanExecute(null));
        }

        [TestMethod]
        public void TestMaxLengthTweet() {
            ITwitterService twitterService = new MockTwitterService();
            IEventAggregator eventAggregator = new MockEventAggregator();

            TweetBoxViewModel vm = new TweetBoxViewModel(twitterService, eventAggregator);

            vm.Tweet = new String('-', 140);
            Assert.IsTrue(vm.RemainingLength == 0);
            Assert.IsTrue(vm.PostCommand.CanExecute(null));
        }

        [TestMethod]
        public void TestTooLongTweet() {
            ITwitterService twitterService = new MockTwitterService();
            IEventAggregator eventAggregator = new MockEventAggregator();

            TweetBoxViewModel vm = new TweetBoxViewModel(twitterService, eventAggregator);

            bool raisedError = false;
            try {
                vm.Tweet = new String('-', 141);
            }
            catch {
                raisedError = true;
            }

            Assert.IsTrue(raisedError);
            Assert.IsTrue(vm.RemainingLength == -1);
            Assert.IsTrue(vm.PostCommand.CanExecute(null) == false);
        }

        [TestMethod]
        [Asynchronous]
        public void TestSuccessfulPost() {
            ITwitterService twitterService = new MockTwitterService(App.Current.SyncContext);
            IEventAggregator eventAggregator = new MockEventAggregator();

            TweetBoxViewModel vm = new TweetBoxViewModel(twitterService, eventAggregator);

            vm.Tweet = "Test Post";
            vm.PostCommand.Execute(null);

            EnqueueDelay(1000);
            EnqueueCallback(() => Assert.IsTrue(((MockTwitterService)twitterService).WasPostCalled));
            EnqueueCallback(() => Assert.IsTrue(vm.Tweet.Length == 0));
            EnqueueTestComplete();
        }
    }
}
