﻿// TwitterService.cs
//

using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.Configuration;
using System.Linq;
using System.Text.RegularExpressions;
using System.Web.DomainServices;
using System.Web.Ria;
using LinqToTwitter;

namespace TwitterBug {

    [EnableClientAccess]
    public class TwitterService : DomainService {

        private TwitterContext Context {
            get {
                string userName = ConfigurationManager.AppSettings["twitterUserName"];
                string password = ConfigurationManager.AppSettings["twitterPassword"];

                return new TwitterContext(userName, password);
            }
        }

        public override bool UsesUtcDateTimes {
            get {
                return true;
            }
        }

        public IEnumerable<Tweet> GetTweets(TweetKind tweetKind) {
            List<Status> statusList = null;

            try {
                switch (tweetKind) {
                    case TweetKind.User:
                        statusList = Context.Status.Where(s => s.Type == StatusType.User).ToList();
                        break;
                    case TweetKind.Friends:
                        statusList = Context.Status.Where(s => s.Type == StatusType.Friends).ToList();
                        break;
                    case TweetKind.Mentions:
                        statusList = Context.Status.Where(s => s.Type == StatusType.Mentions).ToList();
                        break;
                }
            }
            catch {
            }

            if (statusList != null) {
                return from s in statusList.AsQueryable()
                       orderby s.CreatedAt descending
                       select new Tweet {
                           ID = s.ID,
                           Date = DateTime.SpecifyKind(s.CreatedAt, DateTimeKind.Utc),
                           Text = s.Text,
                           ScreenName = s.User.ScreenName,
                           ImageUrl = s.User.ProfileImageUrl,
                           Kind = tweetKind
                       };
            }

            return null;
        }

        #region Profile Query Method
        public TwitterProfile GetTwitterProfile() {
            string userName = ConfigurationManager.AppSettings["twitterUserName"];

            User user = Context.User.Where(u => (u.Type == UserType.Show) && (u.ScreenName == userName)).FirstOrDefault();
            return new TwitterProfile {
                ID = user.ID,
                Status = user.Status.Text,
                Updates = (int)user.StatusesCount,
                Friends = (int)user.FriendsCount,
                Followers = (int)user.FollowersCount,
                ImageUrl = user.ProfileImageUrl.Replace("_normal", "_bigger"),
                Name = user.Name,
                ScreenName = user.ScreenName
            };
        }
        #endregion

        #region Search Query Method
        [Query(IsComposable = false)]
        public IEnumerable<Tweet> SearchTweets(string text, int pageIndex) {
            Search searchResult =
                Context.Search.
                Where(s => s.Type == SearchType.Search &&
                      s.Query == text &&
                      s.ShowUser == "true").
                ToList().FirstOrDefault();

            if (searchResult != null) {
                IEnumerable<Tweet> tweetQuery =
                    from entry in searchResult.Entries
                    select new Tweet {
                        ID = entry.ID.Substring(entry.ID.IndexOf(':') + 1),
                        ScreenName = entry.Author.URI.Substring(entry.Author.URI.LastIndexOf('/') + 1),
                        Text = entry.Content,
                        Date = DateTime.SpecifyKind(entry.Published, DateTimeKind.Utc),
                        ImageUrl = entry.Image
                    };
                List<Tweet> tweets = tweetQuery.ToList();
                foreach (Tweet t in tweets) {
                    t.Text = Regex.Replace(t.Text, "<.*?>", String.Empty).Substring(t.ScreenName.Length + 2);
                }

                return tweets;
            }

            return null;
        }
        #endregion

        #region Status Query/Insert Methods
        public TwitterStatus GetStatus() {
            return null;
        }

        [Insert]
        public void PostStatus(TwitterStatus status) {
            Status twitterStatus = Context.UpdateStatus(status.Text);
            status.ID = twitterStatus.ID;
        }
        #endregion
    }
}
