﻿// BouncingPlane.cs
//

using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Input;
using System.Windows.Interactivity;
using System.Windows.Media;
using System.Windows.Media.Animation;

namespace TwitterBug {
    
    public class BouncingPlace : Behavior<FrameworkElement> {

        private DoubleAnimationUsingKeyFrames BuildKeyFrames(double mass, double springConstant, double dampening, double forceMultiplier, double directionalScalar, double offset) {
            // In milliseconds
            int sample = 75;
            int duration = 10000;

            // Declare physics constants
            double lamda = dampening / (2 * Math.Sqrt(springConstant * mass));
            double omega_not = Math.Sqrt(springConstant / mass);
            double omega_d = omega_not * Math.Sqrt(1.0 - Math.Pow(lamda, 2.0));
            double amplitude = directionalScalar * forceMultiplier;

            DoubleAnimationUsingKeyFrames animation = new DoubleAnimationUsingKeyFrames();
            for (int i = 0; i < (duration / sample); i++) {
                LinearDoubleKeyFrame frame = new LinearDoubleKeyFrame();
                frame.KeyTime = KeyTime.FromTimeSpan(TimeSpan.FromMilliseconds(i * sample));
                frame.Value = offset + amplitude * Math.Exp(-1 * lamda * omega_not * (double)sample * i / 1000) * Math.Sin(omega_d * i * sample / 1000);
                animation.KeyFrames.Add(frame);
            }

            return animation;
        }

        protected override void OnAttached() {
            AssociatedObject.Projection = new PlaneProjection() {
                CenterOfRotationX = 0.5,
                CenterOfRotationY = 0.5
            };
            AssociatedObject.MouseLeftButtonDown += OnMouseLeftButtonDown;
            base.OnAttached();
        }

        protected override void OnDetaching() {
            AssociatedObject.MouseLeftButtonDown -= OnMouseLeftButtonDown;
            base.OnDetaching();
        }

        private void OnMouseLeftButtonDown(object sender, MouseButtonEventArgs args) {
            if (((UIElement)args.OriginalSource) == null)
                return;

            // Get click position relative to the clicked object
            Point p = args.GetPosition((UIElement)args.OriginalSource);

            // Determine simple moment arm and direction against clicked object on both x & y axes.
            double directionalScalarY = (p.Y - (AssociatedObject.RenderSize.Height / 2)) / (AssociatedObject.RenderSize.Height / 2);
            double directionalScalarX = ((AssociatedObject.RenderSize.Width / 2) - p.X) / (AssociatedObject.RenderSize.Width / 2);

            Storyboard sb = new Storyboard();

            DoubleAnimationUsingKeyFrames da = BuildKeyFrames(1.0, 80.0, 1.0, 35.0, directionalScalarX, 5.0);
            Storyboard.SetTarget(da, AssociatedObject.Projection);
            Storyboard.SetTargetProperty(da, new PropertyPath("RotationY"));
            sb.Children.Add(da);

            da = BuildKeyFrames(1.0, 80.0, 1.0, 35.0, directionalScalarY, 5.0);
            Storyboard.SetTarget(da, AssociatedObject.Projection);
            Storyboard.SetTargetProperty(da, new PropertyPath("RotationX"));
            sb.Children.Add(da);

            sb.Begin();
        }
    }
}
