﻿// ProfileBoxViewModel.cs
//

using System;
using System.ComponentModel.Composition;
using PatternsFramework;
using TwitterBug;
using TwitterBug.Services;

namespace TwitterBug.Models {

    [Export]
    public class ProfileBoxViewModel : ViewModelBase {

        private ITwitterService _twitterService;
        private IEventAggregator _eventAggregator;

        private TwitterProfile _profile;
        private bool _loading;

        [ImportingConstructor]
        public ProfileBoxViewModel(ITwitterService twitterService, IEventAggregator eventAggregator) {
            _twitterService = twitterService;

            _eventAggregator = eventAggregator;
            _eventAggregator.Subscribe<TweetEventArgs>(OnTweetEvent);
        }

        public TwitterProfile Profile {
            get {
                if ((_profile == null) && (_loading == false)) {
                    LoadProfile();
                }
                return _profile;
            }
        }

        private void LoadProfile() {
            _loading = true;
            _twitterService.GetProfile(delegate(TwitterProfile profile, object state) {
                _profile = profile;
                _loading = false;
                RaisePropertyChanged("Profile");
            }, null);
        }

        private void OnTweetEvent(TweetEventArgs e) {
            LoadProfile();
        }
    }
}
