﻿// TweetBoxViewModel.cs
//

using System;
using System.ComponentModel.Composition;
using PatternsFramework;
using System.Windows.Input;
using TwitterBug;
using TwitterBug.Services;

namespace TwitterBug.Models {

    [Export]
    public class TweetBoxViewModel : ViewModelBase {

        private int MaximumTweetLength = 140;

        private ITwitterService _twitterService;
        private IEventAggregator _eventAggregator;

        private DelegateCommand _postCommand;
        private string _tweet;

        [ImportingConstructor]
        public TweetBoxViewModel(ITwitterService twitterService, IEventAggregator eventAggregator) {
            _twitterService = twitterService;
            _eventAggregator = eventAggregator;

            _postCommand = new DelegateCommand(Post, GetCanPost);
        }

        public int RemainingLength {
            get {
                return MaximumTweetLength - Tweet.Length;
            }
        }

        public string Tweet {
            get {
                return _tweet ?? String.Empty;
            }
            set {
                _tweet = value ?? String.Empty;
                RaisePropertyChanged("Tweet");
                RaisePropertyChanged("RemainingLength");
                _postCommand.RaiseCanExecuteChanged();

                if (_tweet.Length > MaximumTweetLength) {
                    throw new ArgumentException("Your tweet can be at most 140 characters in length.");
                }
            }
        }

        public ICommand PostCommand {
            get {
                return _postCommand;
            }
        }

        public event EventHandler TweetPosted;

        private void Post() {
            _twitterService.PostTweet(Tweet, delegate(bool result, object state) {
                if (result) {
                    _eventAggregator.Publish<TweetEventArgs>(new TweetEventArgs());
                    if (TweetPosted != null) {
                        TweetPosted(this, EventArgs.Empty);
                    }

                    Tweet = String.Empty;
                }
            }, null);
        }

        private bool GetCanPost() {
            return (Tweet.Length > 0) && (Tweet.Length <= MaximumTweetLength);
        }
    }
}
