﻿// ServerTwitterService.cs
//

using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.Linq;
using System.Windows.Ria.Data;
using TwitterBug;

namespace TwitterBug.Services {

    [Export(typeof(ITwitterService))]
    public class ServerTwitterService : ITwitterService {

        private TwitterContext _context;

        public ServerTwitterService() {
            _context = new TwitterContext();
        }

        #region ITwitterService Members

        IEnumerable<Tweet> ITwitterService.Tweets {
            get {
                return _context.Tweets;
            }
        }

        void ITwitterService.GetProfile(Action<TwitterProfile, object> callback, object userState) {
            _context.Load(_context.GetTwitterProfileQuery(),
                          delegate(LoadOperation<TwitterProfile> loadOperation) {
                              TwitterProfile profile = null;

                              if (loadOperation.HasError == false) {
                                  profile = loadOperation.Entities.FirstOrDefault();
                              }

                              callback(profile, userState);
                          }, null);
        }

        void ITwitterService.PostTweet(string tweet, Action<bool, object> callback, object userState) {
            _context.TwitterStatus.Add(new TwitterStatus() {
                Text = tweet
            });

            _context.SubmitChanges(delegate(SubmitOperation submitOperation) {
                if (submitOperation.HasError) {
                    callback(false, userState);
                }
                else {
                    callback(true, userState);
                }
            }, null);
        }

        void ITwitterService.LoadTweets(TweetKind tweetKind, Action<bool, object> callback, object userState) {
            EntityQuery<Tweet> tweetsQuery = _context.GetTweetsQuery(tweetKind);

            _context.Load(tweetsQuery, delegate(LoadOperation<Tweet> loadOperation) {
                callback(loadOperation.HasError == false, userState);
            }, null);
        }
        #endregion
    }
}
